//-----------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	This example extends the concept from MEventMessage2 but this time
/// 		it's a little bit more exciting - we can access joypad data within
/// 		Maya and wang it on the screen. This is quite a nice example of where
/// 		Idle callbacks are useful. 
/// 			Whilst this is not exactly a 'mocap' plugin, it would probably
/// 		be possible to capture the data if autokeyframe was set to true.
/// 		Otherwise I guess a node that has to dirty all connections and then
/// 		force the surface every idle cycle (hmmm).
///
//-----------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MEventMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MPoint.h>
#include <maya/M3dView.h>
#include <time.h>
#include <stdio.h>
#include <fstream>
#include "Joypad.h"
#include "FrameTimer.h"
#include <GL/gl.h>
#include <GL/glu.h>

#define LEVEL_SIZE 15


/// Having created a callback function within maya, we need to store the generated
/// MCallbackId so that we can delete the callback.
///
MCallbackId g_Callback;

/// pointer to the joypad
XInputJoypad* g_pJoypad = 0;

const unsigned int g_Dim=20;

void DrawSquare(int x,int y) {
	glBegin(GL_QUADS);
		glVertex2i(x,y);
		glVertex2i(x+g_Dim,y);
		glVertex2i(x+g_Dim,y+g_Dim);
		glVertex2i(x,y+g_Dim);
	glEnd();
}



void DrawAxes(float value) {

	glPushMatrix();
	glBegin(GL_LINE_LOOP);
		glVertex2i(1,0);
		glVertex2i(39,0);
		glVertex2i(39,100);
		glVertex2i(1,100);
	glEnd();

	glTranslatef(0,value*50.0f + 50.0f,0);
	glBegin(GL_QUADS);
		glVertex2i(1,-2);
		glVertex2i(39,-2);
		glVertex2i(39,2);
		glVertex2i(1,2);
	glEnd();
	glPopMatrix();
}

float g_Position[2] = {0,0};

/// The function that performs our idle processing
///
void IdleFunc( void* /* ignore */ ) {

	SortFrameTimer();

	float dt = FrameTime();


	if(g_pJoypad && UpdateInputState())
	{
		g_Position[0] += 250 * dt * g_pJoypad->Xaxis() / JP_MAX_EXTENT;
		g_Position[1] += 250 * dt * g_pJoypad->Yaxis() / JP_MAX_EXTENT;


		M3dView View = M3dView::active3dView();

		// to draw the selection circle within maya, we need to
		// draw a custom overlay in the viewport in which the tool
		// is being used.
		//
		// This means we have to surround the
		View.beginOverlayDrawing();

			// clear the overlay planes
			View.clearOverlayPlane();

			// Set up the orthographic projection matix
			glMatrixMode( GL_PROJECTION );
			glLoadIdentity();
			gluOrtho2D( 0.0f, View.portWidth(),
						0.0f, View.portHeight() );

			glMatrixMode( GL_MODELVIEW );
			glLoadIdentity();

			DrawSquare(static_cast<int>(g_Position[0]),static_cast<int>(g_Position[1]));

			// loop through each button and draw indicator for it
			//
			glPushMatrix();
				glTranslatef(View.portWidth() - 20 * g_pJoypad->NumButtons() - 5 ,5,0);
				for(unsigned int i=0;i!=g_pJoypad->NumButtons();++i) {

					// draw loop around square
					glBegin(GL_LINE_LOOP);
						glVertex2i(1,1);
						glVertex2i(18,1);
						glVertex2i(18,18);
						glVertex2i(1,18);
					glEnd();

					// if button is pressed, draw a little square in it
					if(g_pJoypad->Button(i))
					{
						glBegin(GL_QUADS);
							glVertex2i(3,3);
							glVertex2i(16,3);
							glVertex2i(16,16);
							glVertex2i(3,16);
						glEnd();
					}
					glTranslatef(20,0,0);
				}
			glPopMatrix();


			glPushMatrix();
				glTranslatef(View.portWidth() - 20 * g_pJoypad->NumButtons() - 5 ,30,0);

				// if drawing x axis
				if(g_pJoypad->HasX()) {
					DrawAxes( g_pJoypad->Xaxis() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}

				// if drawing y axis
				if(g_pJoypad->HasY()) {
					DrawAxes( g_pJoypad->Yaxis() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}

				// if drawing z axis
				if(g_pJoypad->HasZ()) {
					DrawAxes( g_pJoypad->Zaxis() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}

				// if drawing x rotation axis
				if(g_pJoypad->HasXrot()) {
					DrawAxes( g_pJoypad->Xrot() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}

				// if drawing y rotation axis
				if(g_pJoypad->HasYrot()) {
					DrawAxes( g_pJoypad->Yrot() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}

				// if drawing z rotation axis
				if(g_pJoypad->HasZrot()) {
					DrawAxes( g_pJoypad->Zrot() / JP_MAX_EXTENT );
					glTranslatef(40,0,0);
				}
			glPopMatrix();


		// done drawing our overlay on the viewport
		View.endOverlayDrawing();

		glFlush();
	}
}

/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	// to keep our game at a steady speed ;)
	InitFrameTimer();

	// only create callback if input updated OK
	if( InitInput() && NumJoypads() )
	{
		// get a pointer to the joypad
		g_pJoypad = GetJoypad(0);

		/// When creating the callback for the idle function, we need to specify a callback
		/// type. In this case we are using "idle", however it can be anyone of the strings
		/// that `scriptJob -listEvents` returns;
		/// The second parameter is a pointer to the callback function. The third is a user
		/// data parameter (which is explained in MEventMessage2 example). In this case
		/// we can use NULL since we dont want any user data
		///
		g_Callback = MEventMessage::addEventCallback( "idle", IdleFunc, NULL );

	}
	else {
		std::cerr << "Could not initialise joypads\n";
		return MS::kFailure;
	}

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	if(g_pJoypad) {
		/// make sure we delete the callback before the plugin quits !!!
		/// Maya is likely to crash otherwise!
		MEventMessage::removeCallback( g_Callback );
	}

	/// release DX8 or joystick handles for linux
	FreeInput();

	// clear current overlay
	M3dView View = M3dView::active3dView();
	View.beginOverlayDrawing();
		View.clearOverlayPlane();
	View.endOverlayDrawing();


	return MS::kSuccess;
}


